<form class="search">
    <label for="search_input">
        <input
            class="search-input"
            autocomplete="off"
            #globalSearchBox
            name="globalSearchBox"
            [(ngModel)]="searchTerm"
            id="search_input"
            type="text"
            (keyup)="search(globalSearchBox.value)"
            placeholder="{{ placeholderText | translate }}" />
    </label>
</form>
