<div class="row content-wrapper" *ngIf="!loading">
    <div class="col-md-8 md-container pl-1" *ngIf="readme">
        <div class="md-div" [innerHTML]="readme | markdown"></div>
    </div>
    <div *ngIf="!readme">
        <div *ngIf="type === 'CNAI'">
            <div class="col-md-8 md-container pl-1" *ngIf="fileTooLargeStatus">
                {{ 'ARTIFACT.FILE_TOO_LARGE' | translate }}
            </div>
            <div class="col-md-8 md-container pl-1" *ngIf="noReadmeStatus">
                {{ 'ARTIFACT.MODEL_NO_README' | translate }}
            </div>
        </div>
        <div *ngIf="type !== 'CNAI'" class="col-md-8 md-container pl-1">
            {{ 'ARTIFACT.NO_README' | translate }}
        </div>
    </div>
</div>
<div *ngIf="loading" class="clr-row mt-3 center">
    <span class="spinner spinner-md"></span>
</div>
