<div class="clr-row flex-end">
    <div class="card">
        <div class="card-block">
            <h4 class="head">{{ 'STATISTICS.PRO_ITEM' | translate }}</h4>
            <div class="clr-row" *ngIf="isValidSession">
                <div class="clr-col">
                    {{ 'STATISTICS.INDEX_PRIVATE' | translate }}
                </div>
                <div class="clr-col font-weight-700">
                    {{ originalCopy?.private_project_count }}
                </div>
            </div>
            <div class="clr-row">
                <div class="clr-col">
                    {{ 'STATISTICS.INDEX_PUB' | translate }}
                </div>
                <div class="clr-col font-weight-700">
                    {{ originalCopy?.public_project_count }}
                </div>
            </div>
            <div class="clr-row" *ngIf="isValidSession">
                <div class="clr-col">
                    {{ 'STATISTICS.INDEX_TOTAL' | translate }}
                </div>
                <div class="clr-col font-weight-700">
                    {{ originalCopy?.total_project_count }}
                </div>
            </div>
        </div>
    </div>
    <div class="card">
        <div class="card-block">
            <h4 class="head">{{ 'STATISTICS.REPO_ITEM' | translate }}</h4>
            <div class="clr-row" *ngIf="isValidSession">
                <div class="clr-col">
                    {{ 'STATISTICS.INDEX_PRIVATE' | translate }}
                </div>
                <div class="clr-col font-weight-700">
                    {{ originalCopy?.private_repo_count }}
                </div>
            </div>
            <div class="clr-row">
                <div class="clr-col">
                    {{ 'STATISTICS.INDEX_PUB' | translate }}
                </div>
                <div class="clr-col font-weight-700">
                    {{ originalCopy?.public_repo_count }}
                </div>
            </div>
            <div class="clr-row" *ngIf="isValidSession">
                <div class="clr-col">
                    {{ 'STATISTICS.INDEX_TOTAL' | translate }}
                </div>
                <div class="clr-col font-weight-700">
                    {{ originalCopy?.total_repo_count }}
                </div>
            </div>
        </div>
    </div>
    <div class="card" *ngIf="isValidSession">
        <div class="card-block container">
            <h4 class="head">{{ 'QUOTA.QUOTA_USED' | translate }}</h4>
            <div class="storage-used font-weight-700">
                <div>
                    <span class="size-number margin-right-5px">{{
                        getSizeNumber()
                    }}</span
                    ><span>{{ getSizeUnit() }}</span>
                </div>
            </div>
        </div>
    </div>
</div>
