<clr-datagrid>
    <clr-dg-column>{{ 'REPOSITORY.NAME' | translate }}</clr-dg-column>
    <clr-dg-column>{{
        'REPOSITORY.ARTIFACTS_COUNT' | translate
    }}</clr-dg-column>
    <clr-dg-column>{{ 'REPOSITORY.PULL_COUNT' | translate }}</clr-dg-column>
    <clr-dg-row *clrDgItems="let r of repositories" [clrDgItem]="r">
        <clr-dg-cell
            ><a
                href="javascript:void(0)"
                [routerLink]="
                    getLink(r.project_id, r.name || r.repository_name)
                "
                [queryParams]="getQueryParams()"
                >{{ r.name || r.repository_name }}</a
            ></clr-dg-cell
        >
        <clr-dg-cell>{{ r.artifact_count }}</clr-dg-cell>
        <clr-dg-cell>{{ r.pull_count }}</clr-dg-cell>
    </clr-dg-row>
    <clr-dg-placeholder>{{
        'REPOSITORY.PLACEHOLDER' | translate
    }}</clr-dg-placeholder>
    <clr-dg-footer>
        <span *ngIf="repositories?.length"
            >{{ pagination.firstItem + 1 }} - {{ pagination.lastItem + 1 }}
            {{ 'REPOSITORY.OF' | translate }}
        </span>
        {{ repositories?.length }} {{ 'REPOSITORY.ITEMS' | translate }}
        <clr-dg-pagination #pagination [clrDgPageSize]="5"></clr-dg-pagination>
    </clr-dg-footer>
</clr-datagrid>
