<div class="btn-group btn-tag-integration">
    <button
        [disabled]="!hasTagRetentionPermission"
        routerLink="tag-retention"
        routerLinkActive="btn-primary"
        class="btn"
        id="btn-retention">
        {{ 'TAG_RETENTION.TAG_RETENTION' | translate }}
    </button>
    <button
        [disabled]="!hasTagImmutablePermission"
        routerLink="immutable-tag"
        routerLinkActive="btn-primary"
        class="btn"
        id="btn-immutable">
        {{ 'PROJECT_DETAIL.IMMUTABLE_TAG' | translate }}
    </button>
</div>
<router-outlet></router-outlet>
