<div *ngIf="globalMessageOpened">
    <clr-alert
        [clrAlertType]="globalMessage.type"
        [clrAlertAppLevel]="false"
        [(clrAlertClosed)]="!globalMessageOpened"
        (clrAlertClosedChange)="onClose()">
        <clr-alert-item class="flex-center">
            <span class="alert-text">{{ message }}</span>
        </clr-alert-item>
    </clr-alert>
</div>
