<h2 class="custom-h2" sub-header-title>{{ 'CONFIG.LABEL' | translate }}</h2>
<hbr-label
    [scope]="'g'"
    [hasCreateLabelPermission]="true"
    [hasUpdateLabelPermission]="true"
    [hasDeleteLabelPermission]="true"></hbr-label>
