<clr-tooltip>
    <label
        clrTooltipTrigger
        class="label"
        title="{{ label.name }}"
        [ngStyle]="{
            'background-color': labelColor?.color,
            color: labelColor?.textColor,
            border:
                labelColor?.color === '#FFFFFF' ? '1px solid #A1A1A1' : 'none'
        }"
        [style.max-width.px]="labelWidth">
        <span>
            <clr-icon
                *ngIf="hasIcon && label.scope === 'p'"
                shape="organization"></clr-icon>
            <clr-icon
                *ngIf="hasIcon && label.scope === 'g'"
                shape="administrator"></clr-icon>
        </span>
        <span class="label-name">{{ label.name }}</span>
    </label>
    <clr-tooltip-content
        [clrPosition]="'right'"
        *ngIf="withTooltip && label.description">
        <span>{{ label.description }}</span>
    </clr-tooltip-content>
</clr-tooltip>
