<div
    aria-haspopup="true"
    role="tooltip"
    [class.invalid]="dateInvalid"
    class="tooltip tooltip-validation tooltip-sm">
    <input
        clrDate
        type="date"
        #searchTime="ngModel"
        [(ngModel)]="dateInput"
        name="searchTime"
        placeholder="mm/dd/yyyy"
        dateValidator
        (ngModelChange)="doSearch()" />
    <span *ngIf="dateInvalid" class="tooltip-content">
        {{ 'AUDIT_LOG.INVALID_DATE' | translate }}
    </span>
</div>
