<div class="repo-wrapper">
    <div>
        <h3 class="pop-repos">{{ 'REPOSITORY.POP_REPOS' | translate }}</h3>
    </div>
    <div>
        <list-repository-ro [repositories]="topRepos"></list-repository-ro>
    </div>
</div>
