<clr-modal
    class="hidden-tag"
    [(clrModalOpen)]="retagDialogOpened"
    [clrModalStaticBackdrop]="true">
    <h3 class="modal-title">{{ 'REPOSITORY.RETAG' | translate }}</h3>
    <div class="modal-body retag-modal-body">
        <div class="row col-md-12">
            <hbr-image-name-input #imageNameInput></hbr-image-name-input>
        </div>
    </div>
    <div class="modal-footer">
        <button
            type="button"
            [disabled]="
                imageNameInput.projectName.invalid ||
                imageNameInput.repoName.invalid ||
                imageNameInput.notExist ||
                imageNameInput.checkingName
            "
            class="btn btn-primary"
            (click)="onRetag()">
            {{ 'BUTTON.CONFIRM' | translate }}
        </button>
    </div>
</clr-modal>
