<clr-datagrid [clrDgLoading]="loading">
    <clr-dg-column class="history-time">{{
        'TAG.CREATION' | translate
    }}</clr-dg-column>
    <clr-dg-column>{{ 'TAG.COMMAND' | translate }}</clr-dg-column>
    <clr-dg-row
        *clrDgItems="let h of historyList"
        [clrDgItem]="h"
        class="history-item">
        <clr-dg-cell>{{ h.created | harborDatetime : 'short' }}</clr-dg-cell>
        <clr-dg-cell>{{ h.created_by }}</clr-dg-cell>
    </clr-dg-row>
    <clr-dg-footer>{{ historyList.length }} commands</clr-dg-footer>
</clr-datagrid>
