<h3 class="modal-title">{{ 'SIGN_UP.TITLE' | translate }}</h3>
<new-user-form
    isSelfRegistration="true"
    (valueChange)="formValueChange($event)"></new-user-form>
<div>
    <span class="spinner spinner-inline loading-top" [hidden]="!inProgress">
    </span>
    <button
        type="button"
        class="btn btn-outline"
        [disabled]="!canBeCancelled"
        (click)="cancel()">
        {{ 'BUTTON.CANCEL' | translate }}
    </button>
    <button
        type="button"
        class="btn btn-primary"
        [disabled]="!isValid || inProgress"
        (click)="create()">
        {{ 'BUTTON.SIGN_UP' | translate }}
    </button>
</div>
