<span class="container">
    <cds-icon *ngIf="isError()" class="error" shape="error-standard"></cds-icon>
    <cds-icon
        *ngIf="isWarning()"
        size="20"
        class="warning"
        shape="exclamation-triangle"></cds-icon>
    <cds-icon
        *ngIf="isSuccess()"
        class="success"
        shape="success-standard"></cds-icon>
    <span class="ml-5px">{{ timeRemain | translate }}</span>
</span>
