// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package provider

import (
	"github.com/goharbor/harbor/src/pkg/p2p/preheat/models/provider"
)

// Factory is responsible to create a new driver based on the metadata.
type Factory func(instance *provider.Instance) (Driver, error)

// DragonflyFactory creates dragonfly driver
func DragonflyFactory(instance *provider.Instance) (Driver, error) {
	return &DragonflyDriver{instance}, nil
}

// KrakenFactory creates kraken driver
func KrakenFactory(instance *provider.Instance) (Driver, error) {
	return &KrakenDriver{instance}, nil
}
