.reset-modal-title-override {
    font-size: 14px !important;
}

.form-group-override {
    padding-left: 130px !important;
}

.password-body {
    min-height: 130px;
}

.pw-eye {
    margin-left: -24px;
    margin-right: 8px;
    cursor: pointer;
}

.pwd-input {
    padding-right: 26px;
}

.clr-control-label {
    width: 8.6rem !important;
}
