/* stylelint-disable */
.select .optionMore {
    background-color: #bfbaba;
    height: 1.6em;
    font-size: 1.2em;
    cursor: pointer;
    text-align: center;
}

.hideFilter {
    display: none;
}

h4 {
    color: #666;
}

.alert-label {
    color:red;
    font-size: 12px;
}

.endpointSelect {
    width: 270px;
    margin-right: 10px;
}

.filterSelect {
    position: relative;
    width: 315px;
    margin-bottom:0.25rem;
}

.filterSelect clr-icon {
    margin-left: 10px;
    margin-top: 5px;
    margin-right: 15px;
}

.padLeft0 {
    padding-left: 0;
}

.schedule-style {
    display: inline-block;
}

.floatSet {
    display: inline-block;
    width: 82px;
    margin-right: 4px;
}

.projectInput {
    float: left;
    position: relative;
}

.switchIcon {
    width: 20px;
    height: 20px;
    margin-top: 10px;
    margin-left: 10px;
    cursor: pointer;
}

.addEndpoint {
    margin-top: .25em !important;
    padding-left: 2px;
    padding-right: 2px;
    min-width: 58px;
    margin-right: 0
}

.shadow {
    position: absolute;
    top: 8px;
}

.selectBox {
    position: absolute;
    width: 100%;
    height: auto;
    margin-top: -0.25rem;
    border: 1px solid #ccc;
    background-color: white;
    border: 1px solid rgb(0 0 0 / 15%);
    border-right-width: 2px;
    border-bottom-width: 2px;
    border-radius: 6px;
    box-shadow: 0 5px 10px rgb(0 0 0 / 20%);
    z-index: 100;
}

.selectBox ul li {
    list-style: none;
    padding: 3px 20px;
    cursor: pointer;
}

.selectBox ul li:hover {
    color: #262626;
    background-image: linear-gradient(180deg, #f5f5f5 0, #e8e8e8);
    background-repeat: repeat-x;
}

.form-group-override {
    padding-left: 200px;
}

.form-group {
    >label:first-child{
        font-size: 14px;
        width: 7rem;
    }

   .radio {
       label {
           margin-right:10px;
       }
   }
}

.form-select {
    margin-right: 120px;
}

.go-link {
    line-height: 1rem;
    cursor: pointer;
}

.arrowSet{
    display: inline-block;
    position: absolute;
    right: 50px;
    top: 8px;
    transform: rotate(180deg);
}

clr-modal {
    ::ng-deep div.modal-dialog {
        width: 32rem;
    }
}

.deletedDiv {
    text-align: left;
    line-height: 14px;
    color: red;
}

.deletedDiv a{
    margin-left: 10px;
    text-decoration: underline;
    cursor: pointer;
    color: blue;
}

.modal-body-height {
    max-height: 85vh;
}

.filterSelect .warning-icon {
    margin-left: -15px;
}

.mr-t-11{
    margin-top: 11px;
}

.weekday-width {
    width: 104px;
}

.rule-width {
    width: 100%;
}

.form-cron {
    padding-left:3.8rem;
}

.mode-tooltip {
    margin-left: 5px;
}

.override-box {
    flex-direction: row;

    .override-tooltip {
        margin: 3px 0 0 5px;
    }
}

.spinner-position {
    left: 25%;
}

.cron-tooltip {
    color: gray;
    cursor: default;

    .table-box {
        width: 20rem;
    }
}

.display-none{
    display: none;
}

.width-100 {
    width: 100%;
}

.label-text {
    position: relative;
    height: 1.2rem;
    display: flex;
    align-items: center;
    justify-content: left;
    border-bottom: 0.05rem solid;
}

.ellipsis {
    margin-left: 0.2rem;
    font-size: 16px;
    font-weight: 700;
}

.space-between {
    display: flex;
    justify-content: space-between;
}

.flex {
    display: flex;
}

.sub-label {
    flex: 0 0 5rem;
}

.clr-control-label {
    width: 8.6rem;
}

.single-active {
    width: 16rem;
}

.des-tooltip {
    margin-left: 0.5rem;
}

:host::ng-deep.flatten {
    width: 23rem!important;
}

.width-input {
    width: 17rem;
}

.width-name-resource {
    flex: 0 0 12rem;
    width: 12rem;
}

.width-match-exclude {
    width: 5rem;
}

.width-tag-label {
    width: 6rem;
}

.right-align {
    min-width: 204px;
    overflow-y: auto;
    transform: translateX(-9rem) translateY(1.25rem)!important;
}

.dropdown-item {
    min-height: 26px;
}

.no-labels {
    cursor: default;
    padding: 0 0.5rem;
}

.unit {
    width: 4.18rem !important;
}

.speed {
    width: 14rem;
    display: inline-block;

    input {
      width: 12rem;
    }
}

.unit-select {
    width: 3rem;
}

.ml-10px {
    margin-left: 10px;
}

.label-piece {
    display: flex;
    left: .25rem;
}

.label-container{
    position: absolute;
    display: flex;
    align-items: center;
}

.clr-form-control {
    margin-top: 1rem;
}

.flattening {
    margin-top: 0.5rem;
}

.bandwidth {
    margin-top: 0.25rem;
}

.down {
    height: 25px;
    width: 25px;
    right: 0.2rem;
    color: rgb(164 164 164) !important;
    cursor: pointer;
}
.dropdown {
    left: -1.2rem;
    position: relative !important;
}

.label-input {
    padding-right: 1.2rem;
}

.no-extend {
    flex: 1;
}

.cron-input {
    width: 12rem;
}

.event-label {
    width: unset;
}
