// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';
import { SharedModule } from '../shared/shared.module';
import { SignUpComponent } from './sign-up/sign-up.component';
import { SignUpPageComponent } from './sign-up/sign-up-page.component';
import { SignInComponent } from './sign-in/sign-in.component';
import { SignInGuard } from '../shared/router-guard/sign-in-guard-activate.service';
import { TopRepoService } from './sign-in/top-repo/top-repository.service';
import { TopRepoComponent } from './sign-in/top-repo/top-repo.component';

const routes: Routes = [
    {
        path: 'sign-in',
        canActivate: [SignInGuard],
        component: SignInComponent,
    },
];
@NgModule({
    imports: [RouterModule.forChild(routes), SharedModule],
    declarations: [
        SignUpComponent,
        SignUpPageComponent,
        SignInComponent,
        TopRepoComponent,
    ],
    providers: [TopRepoService],
})
export class AccountModule {}
