// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { TestBed, inject } from '@angular/core/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { MemberPermissionGuard } from './member-permission-guard-activate.service';
import { of } from 'rxjs';
import { ErrorHandler } from '../units/error-handler';
import { UserPermissionService } from '../services';

describe('MemberPermissionGuardActivateServiceGuard', () => {
    const fakeUserPermissionService = {
        getPermission() {
            return of(true);
        },
    };
    beforeEach(() => {
        TestBed.configureTestingModule({
            imports: [RouterTestingModule],
            providers: [
                ErrorHandler,
                MemberPermissionGuard,
                {
                    provide: UserPermissionService,
                    useValue: fakeUserPermissionService,
                },
            ],
        });
    });

    it('should ...', inject(
        [MemberPermissionGuard],
        (guard: MemberPermissionGuard) => {
            expect(guard).toBeTruthy();
        }
    ));
});
