version: 0.1
log:
  level: {{level}}
  fields:
    service: registry
storage:
  cache:
    layerinfo: redis
  {{storage_provider_info}}
  maintenance:
    uploadpurging:
{% if purge_upload.enabled %}
      enabled: true
      age: {{ purge_upload.age }}
      interval: {{ purge_upload.interval }}
      {% if purge_upload.dryrun %}
      dryrun: true
      {% else %}
      dryrun: false
      {% endif %}
{% else %}
      enabled: false
{% endif %}
  delete:
    enabled: true
{% if storage_redirect_disabled %}
  redirect:
    disable: true
{% endif %}
redis:
{% if sentinel_master_set %}
  # sentinel hosts with comma
  addr: {{redis_host}}
  sentinelMasterSet: {{sentinel_master_set}}
{% else %}
  addr: {{redis_host}}
{% endif %}
  readtimeout: 10s
  writetimeout: 10s
  dialtimeout: 10s
  password: {{redis_password}}
  db: {{redis_db_index_reg}}
  enableTLS: {{redis_enableTLS}}
  pool:
    maxidle: 100
    maxactive: 500
    idletimeout: 60s
http:
{% if internal_tls.enabled %}
  addr: :5443
  tls:
    certificate: /etc/harbor/tls/registry.crt
    key: /etc/harbor/tls/registry.key
    minimumtls: tls1.2
{% else %}
  addr: :5000
{% endif %}
  secret: placeholder
  debug:
{% if metric.enabled %}
    addr: :{{ metric.port }}
    prometheus:
      enabled: true
      path: {{ metric.path }}
{% else %}
    addr: localhost:5001
{% endif %}
auth:
  htpasswd:
    realm: harbor-registry-basic-realm
    path: /etc/registry/passwd
validation:
  disabled: true
compatibility:
  schema1:
    enabled: true
