// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package auth

import (
	"fmt"
	"net/http"

	"github.com/goharbor/harbor/src/lib/errors"
)

// bearerAuthorizer authorizes the request by adding `Authorization Bearer credential` header
type bearerAuthorizer struct {
	typeID     string
	accessCred string
}

// Authorize requests
func (ba *bearerAuthorizer) Authorize(req *http.Request) error {
	if req != nil && len(ba.accessCred) > 0 {
		req.Header.Add(authorization, fmt.Sprintf("%s %s", ba.typeID, ba.accessCred))
		return nil
	}

	return errors.Errorf("%s: %s", ba.typeID, "missing data to authorize request")
}

// NewBearerAuth create bearer authorizer
func NewBearerAuth(accessCred string) Authorizer {
	return &bearerAuthorizer{Bearer, accessCred}
}
