.option-left {
  padding-left: 16px;
  margin-top: -6px;
}

.option-right {
  padding-right: 16px;
}

.refresh-btn {
  cursor: pointer;
}

.refresh-btn:hover {
  color: #007CBB;
}
/* stylelint-disable */
.rightPos {
  position: absolute;
  z-index: 100;
  right: 35px;
  margin-top: 14px;
  height: 24px;
}

.endpoint-view {
  position: relative;
}

.flex-min-width {
  min-width: 180px;
}

.margin-right-2px {
  margin-right: 2px;
}

.content-top {
  margin-top: 4px;
}
