// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { ScrollManagerService } from './scroll-manager.service';
import { Directive, HostListener, Input } from '@angular/core';

@Directive({
    selector: '[appScrollAnchor]',
})
export class ScrollAnchorDirective {
    @Input('appScrollAnchor') id: string | number;

    constructor(private manager: ScrollManagerService) {}

    @HostListener('click')
    scroll() {
        this.manager.scroll(this.id);
    }
}
