.artifact-icon-div {
  text-align: center;
  width: 2.5rem;
  height: 0.8rem;
}

.artifact-icon {
  height: 0.8rem;
}

.cell {
  display: flex;
  align-items: center;
  width: 100%;
  height: 100%;
}

.margin-left-5 {
  margin-left: 5px;
}

.ml-8 {
  margin-left: 8rem;
}

.x-dash-line {
  position: absolute;
  width: 80px;
  left: -80px;
  border: 0;
  border-top: 2px dotted #a2a9b6;
}

.y-dash-line {
  position: absolute;
  border: 0;
  border-left: 2px dotted #a2a9b6;
  left: -4.2rem;
  bottom: 0
}

.my-action-item {
  font-size: .7rem;
  letter-spacing: normal;
  background: 0 0;
  border: 0;
  cursor: pointer;
  display: block;
  line-height: 1.15rem;
  margin: 0;
  padding: 0.05rem 1.2rem 0;
  text-align: left;
  width: 100%;
}
