.cron-selection {
    display: flex;
    align-items: center;
}

.gc-start-btn {
    width:150px;
    margin-top: 35px;
}

.flex-200 {
    flex: 0 0 200px;
    max-width: 200px;
}

.mt-05 {
    margin-top: 0.5rem;
}

.font-weight-400 {
    font-weight: 400;
}

.center {
    justify-content: center;
    align-items: center;
}

.flex {
    display: flex;
}

.mt-08 {
    margin-top: 0.8rem;
}

.unit-select {
    padding-top: 1px;
    margin-left: 2rem;
}

.max-width-2rem {
    max-width: 2rem;
}

.pt {
    padding-right: 1rem;
}

.p-06 {
    padding-left: .6rem;
    padding-right: .6rem;
}

.float-left {
    float:left;
    margin-right: 1rem;
}
