// Variables for dark theme should be defined here.
@import "../../node_modules/@clr/ui/clr-ui-dark.min.css";

$mode-background-color: rgb(27 42 50);
$mode-background-color1: $mode-background-color !important;
$mode-background-color2: $mode-background-color;
$mode-background-color3: #17242b;
$mode-font-color1: #acbac3;
$mode-font-color3: $mode-font-color1;
$mode-font-color4: $mode-font-color1;
$mode-font-color5: $mode-font-color1;
$mode-font-color2: #eaedf0;
$mode-link-color1: #4aaed9;
$mode-link-color2: $mode-link-color1;
$select-option-color: #1b2a32;
$select-back-color: #acbac3;
$label-form-color: #212129;
$fill-color1: #ccc;
$right-status-fill-color: white;
$light-color-green: #4cd400;
$color-ddd: #21333b;
$color-f2: transparent;
$color-657b83: unset;
$color-fdf6e3:  transparent;
$header-color: hsl(198deg 100% 15%)!important;
$color-yaml:  #253c46;
$filter-divider-bg-color: #495865;
/* stylelint-disable */
$selectBox-option-hover-bg-color-start: #4aaed9;
$selectBox-option-hover-bg-color-end: #0077b8;
$hbr-result-tip-histogram-inner-bg-color: #21333b;
$harbor-icon-translate-x: 0;
$harbor-icon-drop-shadow-x: 58px;
$command-input-color: #eaedf0;
$command-input-bg-color: #2e4757;
$label-hover-bg-color: #28404d;
$nav-divider-bg-color: #fafafa;
$label-color-input: #ddd;
$central-block-loading-bg-color: rgb(0 0 0 / 50%);
$back-link-color: #4aaed9!important;
$radio-button-checked: #4aaed9;
$input-autofill-bg-color: #1b2a32;
$input-autofill-color: #eaedf0;
$pull-command-icon-color: #4aaed9;
$pull-command-icon-hover-color: #007CBB;
$select-all-for-dropdown-color: #4aaed9;
$normal-border-color: #acbac3;
$text-color-job-service-dashboard: #49aeda;
$datagrid-numeric-filter-input-bg-color: #21333b;
$alert-info-bg-color: none;
@import "./common.scss";
