.option-right {
    padding-right: 16px;
}

.refresh-btn {
    cursor: pointer;
}

.refresh-btn:hover {
    color: #007CBB;
}

.row-right {
    padding-right: 60px;
}

.log-top {
    top: 12px;
}

.check-span {
    display: inline-block;
    width: 16px;
}

.display-f{
    display: flex;
}

.reverse-row {
    display: flex;
    flex-direction: row-reverse;
    align-items: baseline;
}

.flex-items-xs-right {
    display: flex;
    align-items: baseline;

    ::ng-deep {
        clr-date-container{
            margin-top: 0;
        }
    }
}

.width-140 {
    width: 140px;
}