.sub-header-title {
  margin-top: 12px;
  margin-bottom: 12px;
}

.subnav {
  .nav {
    padding-left: 0;
  }
}

.role-label {
  color: #CCC;
  font-size: 14px;
  font-style: italic;
  letter-spacing: 0.01em;
}
/* stylelint-disable */
.backStyle{
  color: #007cbb;
  font-size: 12px;
  cursor: pointer;
}
