// Code generated by mockery v2.53.3. DO NOT EDIT.

package orm

import (
	orm "github.com/beego/beego/v2/client/orm"
	mock "github.com/stretchr/testify/mock"
)

// Creator is an autogenerated mock type for the Creator type
type Creator struct {
	mock.Mock
}

// Create provides a mock function with no fields
func (_m *Creator) Create() orm.Ormer {
	ret := _m.Called()

	if len(ret) == 0 {
		panic("no return value specified for Create")
	}

	var r0 orm.Ormer
	if rf, ok := ret.Get(0).(func() orm.Ormer); ok {
		r0 = rf()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(orm.Ormer)
		}
	}

	return r0
}

// NewCreator creates a new instance of Creator. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewCreator(t interface {
	mock.TestingT
	Cleanup(func())
}) *Creator {
	mock := &Creator{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}
