// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { ScrollManagerService } from './scroll-manager.service';
import { Directive, ElementRef, Input, OnDestroy, OnInit } from '@angular/core';

@Directive({
    selector: '[appScrollSection]',
})
export class ScrollSectionDirective implements OnInit, OnDestroy {
    @Input('appScrollSection') id: string | number;

    constructor(
        private host: ElementRef<HTMLElement>,
        private manager: ScrollManagerService
    ) {}

    ngOnInit() {
        this.manager.register(this);
    }

    ngOnDestroy() {
        this.manager.remove(this);
    }

    scroll() {
        this.host.nativeElement.scrollIntoView({
            behavior: 'smooth',
        });
    }
}
