
.custom-add-button {
    font-size: 12px;
    margin-left: -12px;
}

.filter-icon {
    position: relative;
    right: -12px;
}

.filter-pos {
    float: right;
    margin-right: 24px;
    position: relative;
    top: 10px;
}

.action-panel-pos {
    position: relative;
    padding-left: 12px;
    margin-top: 12px;
}

.refresh-btn {
    position: absolute;
    right: 6px;
    top: 17px;
    cursor: pointer;
}

.refresh-btn:hover {
    color: #007CBB;
}

.hide-create {
    visibility: hidden !important;
}
/* stylelint-disable */
.rightPos {
    position: absolute;
    right: 20px;
    height:32px;
    z-index: 100;
}

.relative {
    position: relative;
}
