.form-group-override {
    padding-left: 170px !important;
}

.form-group-label-override {
    font-size: 14px;
    font-weight: 400;
}

.sub-label-for-input {
    position: absolute;
    top: 26px;
    font-size: 10px;
    font-weight: 400;
    line-height: 12px;
    left: 170px;
}

.body-format {
    overflow-y: hidden;
}

.loading-top {
    top: 8px;
}