.label-config {
  position: relative;
  margin-top: 30px;
  /* stylelint-disable */
    .rightPos {
      position: absolute;
      z-index: 100;
      right: 35px;
      height: 24px;

      .option-right {
        padding-right: 16px;

          .refresh-btn {
            cursor: pointer;

              :hover {
              color: #007CBB;
            }
          }
      }    
  }

  hbr-label-piece {
    max-width: 100%;
    display: flex;
  }
}



