.form-group-override {
    padding-left: 0 !important;
}

.vertical-center {
    display: flex;
    align-items: center;
}

.normal-wrapper {
    > span:not(:first-child) {
        margin-right: 18px;
    }
}

.section-title {
    font-size: 14px !important;
    font-weight: 600 !important;
}

.btn-font {
    font-size: 12px !important;
}

.clr-dropdown-override {
    margin-top: -8px;

    .dropdown-namespace {
        min-width: 300px;

            .namespace {
                margin-left: 24px;
            }
    }
}

.button-group {
    display: flex;
    align-items: center;

    .btn-scan-right {
        button{
            width: 160px;
            margin-bottom: 0;
            margin-top: 41px;
       }
    }
}

.update-time {
    font-size: .5417rem;
    width: 200px;
    padding-right: 1rem;
}

.font-style {
    color: #000;
}

.margin-left-5 {
    margin-left: 5px;
}

.margin-top-16px {
    margin-top: 16px;
}

.v-mid {
    vertical-align: middle;
}

.flex-200 {
    flex: 0 0 200px;
}

.pt-4 {
    padding-top: 1rem;
}

.total {
    text-align: right;
    line-height: 15px;
    height: 15px;
    font-size: 10px;
    margin-top: 7px;
    margin-bottom: 3px;
}

.info {
    height: 18px;
    line-height: 18px;
    font-size: 14px;
}

.container {
    height: 18px;
    display: flex;
    background-color:#eee;
}

.state {
    flex: 1;
    padding-left: 3px;
}

.value {
    flex: 3;
}

.float-left {
    float: left;
}

.display-flex {
    display: flex;
}

.error {
    background-color: #e62700;
}

.finished {
    background-color: #62a420;
}

.in-progress {
    background-color: #0079b8;
}

.abort {
    background-color: #cccc15;
}

.badge {
    min-width: 30px;
}

.info-tips-icon {
    color: grey;
}

.info-tips-icon:hover {
    color: #007CBB;
}

.replication-config {
    margin-top: 0;
    margin-bottom: 0;
}

.replication-text {
    font-size: 14px;
    font-weight: 600;
}

.replication-tooltip {
    top: -8px;
}

.margin-top-3px {
    margin-top: 3px;
}
