// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';
import { GcComponent } from './gc-page/gc/gc.component';
import { GcHistoryComponent } from './gc-page/gc/gc-history/gc-history.component';
import { SharedModule } from '../../../shared/shared.module';
import { SetJobComponent } from './audit-log-purge/set-job/set-job.component';
import { ClearingJobComponent } from './clearing-job.component';
import { PurgeHistoryComponent } from './audit-log-purge/history/purge-history.component';

const routes: Routes = [
    {
        path: '',
        component: ClearingJobComponent,
        children: [
            {
                path: 'gc',
                component: GcComponent,
            },
            {
                path: 'audit-log-purge',
                component: SetJobComponent,
            },
            {
                path: '',
                redirectTo: 'gc',
                pathMatch: 'full',
            },
        ],
    },
];
@NgModule({
    imports: [SharedModule, RouterModule.forChild(routes)],
    declarations: [
        GcComponent,
        GcHistoryComponent,
        ClearingJobComponent,
        SetJobComponent,
        PurgeHistoryComponent,
    ],
})
export class ClearingJobModule {}
