// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package tag

import (
	"github.com/goharbor/harbor/src/pkg/tag/model/tag"
)

// Tag is the overall view of tag
type Tag struct {
	tag.Tag
	Immutable bool `json:"immutable"`
}

// Option is used to specify the properties returned when listing/getting tags
type Option struct {
	WithImmutableStatus bool
}
