clr-modal {
    ::ng-deep div.modal-dialog {
        width: unset;
    }

    .rename-tool {
        .btn {
            margin-right: 6px;
            margin-left: 5px;
            padding-left: 3px;
            padding-right: 3px;
        }

    bottom: 9px;
    }

    .label-inner-text{
        margin: 0;
    }

    .cli-secret {
        align-items: center;

        .btn-padding-less {
            padding-left: 5px;
            padding-right: 5px;
            margin-left: 5px;
        }

        .input-cli {
            width: 9.5rem;
        }
    }

    .hidden-generate-cli {
        font-size: 30px;
        color: grey;
        margin-left: 0.5rem;
        cursor: pointer;
        line-height: 30px;
    }
}

.reset-cli-form {
    width: 19.5rem;
}

.display-flex {
    display: flex;
}

.set-btns {
    display: flex;
}

:host::ng-deep.upload-secret {
    width: 24.5rem;
}

.pwd-input {
    padding-right: 26px;
}

.pw-eye {
    margin-left: -20px;
    margin-right: 8px;
    cursor: pointer;
}
