$input-and-select-width: 8rem;



.flex {
  display: flex;
}

.clr-control-container {
  align-items: center;
}

.clr-select {
  min-width: $input-and-select-width;
  margin-top: 1px;
}

.clr-input-wrapper {
  min-width: $input-and-select-width;
}

.clr-control-label {
  width: 4rem !important;
}

.plus {
  margin-left: 2rem;
  background-color: green;
  border-radius: 50%;
  color: white;
  cursor: pointer;
}

.minus {
  margin-left: 1rem;
  background-color: red;
  border-radius: 50%;
  color: white;
  cursor: pointer;
}

.clr-form {
  padding-left: 0.75rem;
}

.disabled {
  cursor: not-allowed;
  background-color: gray;
}

.clr-row {
  align-items: baseline;

}
