.modal {
    background-color: rgb(80 80 80);

    .body-message {
        margin-top: 10px;
    }

    .modal-header {
        .close {
            margin-right: 0.2rem
        }
    }
}

.oidc-header-text{
    color:rgb(94 94 94);
}

.close-error {
    padding-right:0;
}
