# all value should be string
SCANNER_ENDPOINT = "http://<scanner_ip>:8081"
ES_ENDPOINT =  "<elastic_search_ip>:9200" 
WEBHOOK_ENDPOINT =  "<webhook_server_ip>:8084" 
DISTRIBUTION_ENDPOINT =  "http://<dragonfly_server_ip>:8080" 
DOCKER_USER =   "<docker_user_name>" 
DOCKER_PWD =  "<docker_password>" 
DRAGONFLY_AUTH_TOKEN =  "<dragonfly_auth_token>"
ip = "<harbor ip address or fqdn>"

# fixed value, do not change
LOCAL_REGISTRY =  "registry.goharbor.io" 
LOCAL_REGISTRY_NAMESPACE =   "harbor-ci" 
HARBOR_PASSWORD =   "Harbor12345"
OIDC_HOSTNAME =  "nightly-oidc.harbor.io" 
http_get_ca =  "true" 
ip1 = ""
