  .form-group-label-override {
    font-size: 14px;
    font-weight: 400;
  }

  .form-height {
    height: 30px;
  }
  /* stylelint-disable */
  .inputWidth {
    width: 216px;
  }

  .display-none {
    display: none
  }

  .selectBox{
    position: absolute;
    width: 173px;
    height: auto;
    border: 1px solid #ccc;
    background-color: white;
    border-right-width: 2px;
    border-bottom-width: 2px;
    border-radius: 6px;
    box-shadow: 0 5px 10px rgb(0 0 0 / 20%);
    z-index: 100;

    ul {
      overflow-y: auto;
      max-height: 260px;

      li {
        list-style: none;
        padding: 0 18px;
        height:20px;
        line-height: 18px;
        cursor: pointer;
      }

      li:hover {
        color: #262626;
        background-image: linear-gradient(180deg,#f5f5f5 0,#e8e8e8);
        background-repeat: repeat-x;
      }
    }
  }

  .position-relative {
    position: relative;
  }

  .overflow-visible {
    overflow: visible;
  }

  .down {
    position: absolute;
    top: .2917rem;
    right: 0;
    margin: 0;
    width: 12px;
    height: 12px;
    color: rgb(164 164 164);
    cursor: pointer;
  }

  .endpoint-input {
    padding-right: 21px;
  }

  .destination-insecure-label {
    display: flex;
    align-items: center;
  }
