// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package auth

const (
	// AuthModeNone means no auth required
	AuthModeNone = "NONE"
	// AuthModeBasic is basic mode
	AuthModeBasic = "BASIC"
	// AuthModeOAuth is OAuth mode
	AuthModeOAuth = "OAUTH"
	// AuthModeCustom is custom mode
	AuthModeCustom = "CUSTOM"
)

// Credential stores the related data for authorization.
type Credential struct {
	Mode string

	// Keep the auth data.
	// If authMode is 'BASIC', then 'username' and 'password' are stored;
	// If authMode is 'OAUTH', then 'token' is stored'
	// If authMode is 'CUSTOM', then 'header_key' with corresponding header value are stored.
	Data map[string]string
}
