.label-info {
    margin: 0 !important;
    padding: 0 !important;
    margin-top: -5px !important;
}

.spinner-pos {
    margin-right: 0 !important;
    top: 2px;
}

.pw-eye {
    margin-left: -24px;
    margin-right: 8px;
    cursor: pointer;
}

.pwd-input {
    padding-right: 26px;
    width: 228px;
}
