// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package model

import (
	"github.com/goharbor/harbor/src/pkg/quota/types"
	"github.com/goharbor/harbor/src/server/v2.0/models"
)

// ResourceList model
type ResourceList struct {
	types.ResourceList
}

// ToSwagger converts the resource list to the swagger model
func (rl *ResourceList) ToSwagger() models.ResourceList {
	result := make(map[string]int64, len(rl.ResourceList))

	for name, value := range rl.ResourceList {
		result[string(name)] = value
	}

	return result
}

// NewResourceList new resource list instance
func NewResourceList(rl types.ResourceList) *ResourceList {
	return &ResourceList{ResourceList: rl}
}
