.color-97 {
  color: #979797;
}

.over-line {
  height: 15px;
  line-height: 15px;
  font-size: 10px;
}

.height-72 {
  height: 72px;
}

.height-95 {
  height: 95px;
}

.display-none {
  display: none;
}

.untagged {
  display: flex;
  justify-content: space-around;
}

.modal-body {
  overflow-y: hidden;
}

.tootip {
  display: block;
  line-height: .9rem;
}

.font-size-13 {
  font-size: 13px;
}

.margin-top-5px {
  margin-top: 5px;
}
