.about-version {
    font-size: 14px;
    color: #565656;
    font-weight: 500;
}

.dialog-body {
    display:flex;
    padding-left: 0;
}

.content {
    margin:0 10px 10px;
    outline: none;
    width: 470px;
}

.harbor-icon {
    width: 56px;
    filter: drop-shadow(rgb(0 0 0) 58px 2px);
}

.harbor-logo-black {
    width: 60px;
    overflow: hidden;
}