clr-signpost {
    button {
        min-width: 0;
        margin: 0;
        height: 24px;
        vertical-align: super;
        line-height: 6px;
    }

    clr-signpost-content {
        ::ng-deep {
            .signpost-content-header {
                display: none;
            }

            .signpost-content-body {
                padding-bottom: 6px;
                padding-top: 6px;
            }
        }
    }
}