.arrow-block a{
    cursor: pointer;
    color: #007cbb;
    font-size: 16px;
    padding: 5px;
    border-radius: 2px;
    margin-right: 5px;
}

.back-icon {
    color: #007cbb;
    font-size: 16px;
}

.margin-top-5px {
    margin-top: 5px;
}

.center {
    justify-content: center;
    align-items: center;
}

.center-align-items {
    display: flex;
    align-items: center;
}

.artifact-icon {
    width: 25px;
}

.margin-left-10px {
    margin-left: 10px;
}

.margin-right-5px {
    margin-right: 5px;
}
