// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package exporter

import (
	"net"
	"net/http"
	"net/url"
	"strconv"
)

var hbrCli *HarborClient

// HarborClient is client for request harbor
type HarborClient struct {
	HarborScheme string
	HarborHost   string
	HarborPort   int
	*http.Client
}

func (hc HarborClient) harborURL(p string) url.URL {
	return url.URL{
		Scheme: hc.HarborScheme,
		Host:   net.JoinHostPort(hc.HarborHost, strconv.Itoa(hc.HarborPort)),
		Path:   p,
	}
}

// Get ...
func (hc HarborClient) Get(p string) (*http.Response, error) {
	hbrURL := hc.harborURL(p)
	res, err := http.Get(hbrURL.String())
	if err != nil {
		return nil, err
	}
	return res, nil
}

// InitHarborClient initialize the harbor client
func InitHarborClient(hc *HarborClient) {
	hbrCli = hc
}
