.commands-container {
    min-width: 360px;
    max-width: 720px;
    padding-left: 24px;
}

.h5-override {
    display: inline-block !important;
    margin-top: 0 !important;
    margin-bottom: 12px;
}

.commands-section {
    margin-top: 12px;
    margin-bottom: 24px;
}

.info-tips-icon:hover {
    cursor: pointer;
}

.command-title {
    font-size: 14px;
}

.command-input {
    font-size: 14px;
    font-weight: 500;
    border: 0;
}

:host::ng-deep.dropdown-menu {
    min-width: 360px;
    max-width: 720px;
}

.btn-font {
    font-size: 14px !important;
}

.hide{
    display:none;
}
/* stylelint-disable */
.inputTarget {
    width: 1px;
    min-width: 0;
    padding: 0;
    opacity: .01;
}

.dropdown-width {
    min-width: 600px;
}
