:host::ng-deep.modal-dialog {
  width: 30rem;
}

.modal-body {
  outline: none;
  padding-top: 0.8rem;
  overflow-y: visible;
  overflow-x: visible;

.body-label {
  font-size: 14px
}

  .clr-form {
    padding-left: 0;

    .left-label {
      width: 11.5rem;
      font-weight: 100;
      position: relative;
    }

    .mr-3px {
      margin-right: 3px;
    }

    .quota-input {
      width: 2.25rem;
      margin-left: 1rem;
      margin-right: 0.2rem;
    }

    .clr-validate-icon {
      margin-left: -0.6rem;
    }

    .select-div {
      width: 2.5rem;
    }

    .clr-select-wrapper {
      position: absolute;
      right: -8rem;
      top: -0.11rem;
    }

  }

  .clr-form-compact-common {
    .left-label {
      width: 9.5rem;
    }

    .quota-input {
      margin-left: 0;
    }

    .select-div {
      width: 1.6rem;
    }

    .clr-select-wrapper {
      right: -5.5rem;
    }
  }
}

select {
  font-weight: 100;
}

.progress-block {
  width: 8rem;
}

.progress-div {
  position: absolute;
  padding-right: 0.6rem;
  width: 7.5rem;
  top: 0.2rem;
  right: -13.3rem;
}

.progress-label {
  position: absolute;
  right: -4rem;
  margin-top: 0;
  width: 5rem;
  font-weight: 100;
  font-size: 10px;
}
/* stylelint-disable */
::ng-deep {
  .clr-error {
    .clr-validate-icon {
      margin-left: -12px;
    }
  }
}
