// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package auth

// Static handler registry
var knownHandlers = map[string]Handler{
	AuthModeNone:   &NoneAuthHandler{},
	AuthModeBasic:  &BasicAuthHandler{&BaseHandler{}},
	AuthModeCustom: &CustomAuthHandler{&BaseHandler{}},
	AuthModeOAuth:  &TokenAuthHandler{&BaseHandler{}},
}

// GetAuthHandler gets the handler per the mode
func GetAuthHandler(mode string) (Handler, bool) {
	h, ok := knownHandlers[mode]

	return h, ok
}
