.breadcrumb a {
    text-decoration: none;
    cursor: pointer;
    color: #007cbb;
    font-size: 16px;
    padding: 5px;
    border-radius: 2px;
    margin-right: 5px;
}

.back-icon {
    color: #007cbb;
    font-size: 16px;
}
