.label {
    display: flex;
    align-items: center;
    cursor: inherit;
}

.label clr-icon {
    margin-right: 3px;
    margin-top: 3px;
}

.label-name {
    min-width: 0;
    overflow: hidden;
    text-overflow: ellipsis;
}
