// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * For user management
 *
 **
 * class User
 */
export class User {
    user_id?: number;
    username?: string;
    realname?: string;
    email?: string;
    password?: string;
    comment?: string;
    deleted?: boolean;
    role_name?: string;
    role_id?: number;
    sysadmin_flag?: boolean;
    reset_uuid?: string;
    creation_time?: string;
    update_time?: string;
}
export interface LDAPUser {
    ldap_username: string;
    ldap_realname: string;
    ldap_email: string;
}
function LDAPUsertoUser(ldapU: LDAPUser): User {
    let user = new User();
    user.user_id = 0;
    user.username = ldapU.ldap_username;
    user.realname = ldapU.ldap_realname;
    user.email = ldapU.ldap_email;
    return user;
}
export default LDAPUsertoUser;
