.license {
  display: block;
  font-family: monospace;
  word-wrap: break-word;
  white-space: pre-wrap;
  padding: 1em 0;
  font-size: 1rem;
}

.overflow-y-scroll {
  height: 100%;
  width: 100%;
  position: fixed;
  overflow-y: auto;
}
