.duration {
    width: 100%;
    text-align: center;
    font-size: 50px;
    margin: 1rem 0;
    line-height: 50px;
    height: 50px;
}

.card-block {
    height: 9rem;
}


.center {
    display: flex;
    align-items: center;
}

.loading {
    display: flex;
    height: 4rem;
    align-items: center;
    justify-content: center;
}
