.alert-text {
  flex: 0 0 auto !important;
}

.alert-action {
  text-decoration: underline
}

.last {
  position: absolute;
  right: 3rem;
}

.no-underline {
  text-decoration: none;
}

.banner-message {
  min-width: 0;
  word-break: break-word;
}
