.label-form {
    max-width: 100% !important;
}

.clr-control-container {
    margin-bottom: 1rem;
    position: relative;
}

.btn.remove-btn {
    border: none;
    height: 0.6rem;
    line-height: 1;
}

.immutable {
    display: flex;
    align-items: center;
    position: relative;

    .label {
        right: 0;
        margin-right: 0;
    }
}

.datagrid-action-bar {
    margin-top: 0.5rem;
}

.position-ab {
    position: absolute;
}

.white-space-nowrap {
    white-space: nowrap;
}

.pull {
    display: flex;
    flex-flow: row wrap;
    align-content: center;
    justify-content: space-between;
    align-items: center;
}

.pull-label {
    display: flex;
    flex-flow: row wrap;
    gap: 8px;
    align-content: flex-end;
    justify-content: space-between;
    align-items: flex-end;
}

.refresh-btn {
    cursor: pointer;
}

.spinner-tag {
    position: absolute;
    right: -0.7rem;
    top: 1.2rem;
}

.action-bar {
    display: flex;
    align-items: center;
    justify-content: space-between;
}

.right-pos {
    margin-right: 35px;
    display: flex;
    align-items: center;
}

.pull-btn {
    float: right;
    margin-right: -20px;
}
