// Code generated by mockery v2.53.3. DO NOT EDIT.

package export

import (
	context "context"

	artifact "github.com/goharbor/harbor/src/controller/artifact"

	mock "github.com/stretchr/testify/mock"
)

// FilterProcessor is an autogenerated mock type for the FilterProcessor type
type FilterProcessor struct {
	mock.Mock
}

// ProcessLabelFilter provides a mock function with given fields: ctx, labelIDs, arts
func (_m *FilterProcessor) ProcessLabelFilter(ctx context.Context, labelIDs []int64, arts []*artifact.Artifact) ([]*artifact.Artifact, error) {
	ret := _m.Called(ctx, labelIDs, arts)

	if len(ret) == 0 {
		panic("no return value specified for ProcessLabelFilter")
	}

	var r0 []*artifact.Artifact
	var r1 error
	if rf, ok := ret.Get(0).(func(context.Context, []int64, []*artifact.Artifact) ([]*artifact.Artifact, error)); ok {
		return rf(ctx, labelIDs, arts)
	}
	if rf, ok := ret.Get(0).(func(context.Context, []int64, []*artifact.Artifact) []*artifact.Artifact); ok {
		r0 = rf(ctx, labelIDs, arts)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).([]*artifact.Artifact)
		}
	}

	if rf, ok := ret.Get(1).(func(context.Context, []int64, []*artifact.Artifact) error); ok {
		r1 = rf(ctx, labelIDs, arts)
	} else {
		r1 = ret.Error(1)
	}

	return r0, r1
}

// ProcessRepositoryFilter provides a mock function with given fields: ctx, filter, projectIDs
func (_m *FilterProcessor) ProcessRepositoryFilter(ctx context.Context, filter string, projectIDs []int64) ([]int64, error) {
	ret := _m.Called(ctx, filter, projectIDs)

	if len(ret) == 0 {
		panic("no return value specified for ProcessRepositoryFilter")
	}

	var r0 []int64
	var r1 error
	if rf, ok := ret.Get(0).(func(context.Context, string, []int64) ([]int64, error)); ok {
		return rf(ctx, filter, projectIDs)
	}
	if rf, ok := ret.Get(0).(func(context.Context, string, []int64) []int64); ok {
		r0 = rf(ctx, filter, projectIDs)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).([]int64)
		}
	}

	if rf, ok := ret.Get(1).(func(context.Context, string, []int64) error); ok {
		r1 = rf(ctx, filter, projectIDs)
	} else {
		r1 = ret.Error(1)
	}

	return r0, r1
}

// ProcessTagFilter provides a mock function with given fields: ctx, filter, repositoryIDs
func (_m *FilterProcessor) ProcessTagFilter(ctx context.Context, filter string, repositoryIDs []int64) ([]*artifact.Artifact, error) {
	ret := _m.Called(ctx, filter, repositoryIDs)

	if len(ret) == 0 {
		panic("no return value specified for ProcessTagFilter")
	}

	var r0 []*artifact.Artifact
	var r1 error
	if rf, ok := ret.Get(0).(func(context.Context, string, []int64) ([]*artifact.Artifact, error)); ok {
		return rf(ctx, filter, repositoryIDs)
	}
	if rf, ok := ret.Get(0).(func(context.Context, string, []int64) []*artifact.Artifact); ok {
		r0 = rf(ctx, filter, repositoryIDs)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).([]*artifact.Artifact)
		}
	}

	if rf, ok := ret.Get(1).(func(context.Context, string, []int64) error); ok {
		r1 = rf(ctx, filter, repositoryIDs)
	} else {
		r1 = ret.Error(1)
	}

	return r0, r1
}

// NewFilterProcessor creates a new instance of FilterProcessor. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewFilterProcessor(t interface {
	mock.TestingT
	Cleanup(func())
}) *FilterProcessor {
	mock := &FilterProcessor{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}
