/* stylelint-disable */
.form .form-block, form .form-block {
  margin: .5rem 0 2rem;
}

.pw-eye {
  margin-left: -20px;
  margin-right: 8px;
  cursor: pointer;
}

.pwd-input {
  padding-right: 26px;
}
