.filter-icon {
    position: relative;
    right: -20px;
}

.filter-divider {
    display: inline-block;
    height: 16px;
    width: 1px;
    padding-top: 12px;
    padding-bottom: 12px;
    position: relative;
    top: 9px;
    margin-right: 6px;
    margin-left: 6px;
}

.search-btn {
    cursor: pointer;
}

.search-btn:hover {
    color: #007CBB;
}

.filter-input {
    padding-left: 24px;
}
