.has-label {
  max-height: 300px;
  overflow-y: auto;
  margin-top: 10px;
}

.filter-label-input {
  display: flex;
  justify-content: center;
}

.no-labels {
  height: 4rem;
  display: flex;
  align-items: center;
  justify-content: center;
}

.dropdown-item {
  padding: 3px;
  display: flex;
  align-items: center;
}

.check-icon {
  margin-left: 5px;
  margin-right: 5px;
  margin-top: 3px;
}

.clr-form-control {
  margin-top: 0 !important;
}
