@mixin refresh-button {
    cursor: pointer;
    margin-top: 7px;
}

@mixin refresh-button-hover($color) {
    color: $color;
}

@mixin float-right($right-margin) {
    display: flex;
    justify-content: flex-end;
    margin-right: $right-margin;
}

@mixin square($edge) {
    width: $edge;
    height: $edge;
    min-height: $edge;
}
