.action-head-pos {
  padding-right: 18px;
  height: 24px;
  display: flex;
  justify-content: flex-end;
}

.refresh-btn {
  cursor: pointer;
  margin-top: 7px;
}

.margin-left-5 {
  margin-left: 5px;
}

.width-240 {
  min-width: 240px !important;
}

.width-120 {
  min-width: 120px !important;
}

.margin-top-0 {
  margin-top: 0;
}

.margin-top-2 {
  margin-top: 2px;
}

.margin-left-10 {
  margin-left: 10px;
}

.doc {
   font-size: 0.7rem;
}
