.scanner-name {
  height: 1rem;
  display: inline-block;
  padding: 0 .25rem;
  max-height: 1rem;
  font-size: .5417rem;
}

.edit {
  margin-top: -5px;
  margin-left: -8px;
  font-size: 0.58rem;
}

.width-240 {
  width: 240px;
}

.center {
  justify-content: center;
  align-items: center;
}

.clr-form-control {
  margin-top: 0.75rem !important;
}

.name {
  height: 1.75rem;
  line-height: 1.75rem;
}
