<!DOCTYPE HTML>
<html>
<head>
<meta charset="utf-8">
<title>404 - Page Not Found</title>
<style>

*{
	maring: 0;
	padding: 0;
}

a:link{
	color: #1f3759;
	text-decoration: none;
}
a:active{
	color: #1f3759;
	text-decoration: none;
}
a:hover{
	color: #9fb7d9;
	text-decoration: none;
}
a:visited{
	color: #1f3759;
	text-decoration: none;
}

a.underline, .underline{
	text-decoration: underline;
}

#content{
	margin: 0 auto;
	width: 800px;
}

#main-body{
	text-align: center;
}

.status-code {
    font-weight: bolder;
    font-size: 4em;
    vertical-align: middle;
}

.status-text {
    font-weight: bold;
    font-size: 3em;
    margin-left: 10px;
    vertical-align: middle;
}

.status-subtitle {
    font-size: 18px;
}

</style>
</head>
<body>
    <div id="content">             
        <div id="main-body">
			<div>
		        <span class="status-code">404</span>
		        <span class="status-text">Page Not Found</span>
		    </div>
		    <div class="status-subtitle">
		        <p><a href="/harbor" class="underline">Home</a></p>
		    </div>
        </div>
    </div>
</body>
</html>
