.replication-tasks {
    .overview-section {
        .title-wrapper {
            .onback{
                color: #007cbb;
                font-size: 14px;
                cursor: pointer;
            }

            .title-block {
                display: flex;
                align-items: center;

                >div:first-child {
                    min-width: 250px;
                    margin-right: 5px;
                }

                >div:nth-child(2) {
                    width: 150px;

                    span {
                        color: #007cbb;
                        font-size: 12px;
                        margin-left: 10px;
                    }
                }

                .id-divider {
                    display: inline-block;
                    height: 25px;
                    width: 2px;
                    background-color: #ccc;
                    margin: 0 20px -2px;
                }

                .h2-style {
                    display: inline-block;
                }
            }
        }

        .execution-block {
            margin-top: 24px;
            display: flex;
            flex-wrap: wrap;

            .execution-detail-label {
                margin-right: 10px;
                text-align: left;

                .detail-row {
                    display: flex;
                    height: 27px;

                    .common-style {
                        width: 12px;
                        height: 12px;
                        margin-top: 16px;
                    }

                    .num-success {
                        background-color: #308700;
                    }

                    .num-failed {
                        background-color: #C92101;
                    }

                    .num-progress {
                        background-color: #1C5898;
                    }

                    .num-stopped {
                        background-color: #A1A1A1;
                    }

                    .detail-span {
                        flex:0 0 150px;
                        margin: 10px 0 0 10px;
                    }

                    .execution-details {
                        width: 200px;
                        margin: 8px 35px;
                    }
                }
            }

            .executions-detail {
                width: 400px;

                label {
                    display: inline-block;
                    width: 140px;
                }
                /* stylelint-disable */
                >div {
                    margin-top: 10px;
                }
            }
        }
    }

    .tasks-detail {
        margin-top: 65px;

        .action-select {
            padding-right: 18px;
            height: 24px;
            display: flex;
            justify-content: flex-end;

            .filter-tag {
                float: left;
                margin-top: 8px;
            }

            .refresh-btn {
                cursor: pointer;
                margin-top: 7px;
            }

            .refresh-btn:hover {
                color: #007CBB;
            }
        }

        clr-datagrid {
            margin-top: 20px;

            .resource-width {
                width: 150px;
            }
        }
    }
}

.margin-top-16px {
    margin-top: 16px;
}

.button-stop {
    margin-top: 16px;
    margin-left: -5px;
}
