// Code generated by mockery v2.53.3. DO NOT EDIT.

package model

import (
	model "github.com/goharbor/harbor/src/pkg/accessory/model"
	mock "github.com/stretchr/testify/mock"
)

// Accessory is an autogenerated mock type for the Accessory type
type Accessory struct {
	mock.Mock
}

// Display provides a mock function with no fields
func (_m *Accessory) Display() bool {
	ret := _m.Called()

	if len(ret) == 0 {
		panic("no return value specified for Display")
	}

	var r0 bool
	if rf, ok := ret.Get(0).(func() bool); ok {
		r0 = rf()
	} else {
		r0 = ret.Get(0).(bool)
	}

	return r0
}

// GetData provides a mock function with no fields
func (_m *Accessory) GetData() model.AccessoryData {
	ret := _m.Called()

	if len(ret) == 0 {
		panic("no return value specified for GetData")
	}

	var r0 model.AccessoryData
	if rf, ok := ret.Get(0).(func() model.AccessoryData); ok {
		r0 = rf()
	} else {
		r0 = ret.Get(0).(model.AccessoryData)
	}

	return r0
}

// IsHard provides a mock function with no fields
func (_m *Accessory) IsHard() bool {
	ret := _m.Called()

	if len(ret) == 0 {
		panic("no return value specified for IsHard")
	}

	var r0 bool
	if rf, ok := ret.Get(0).(func() bool); ok {
		r0 = rf()
	} else {
		r0 = ret.Get(0).(bool)
	}

	return r0
}

// IsSoft provides a mock function with no fields
func (_m *Accessory) IsSoft() bool {
	ret := _m.Called()

	if len(ret) == 0 {
		panic("no return value specified for IsSoft")
	}

	var r0 bool
	if rf, ok := ret.Get(0).(func() bool); ok {
		r0 = rf()
	} else {
		r0 = ret.Get(0).(bool)
	}

	return r0
}

// Kind provides a mock function with no fields
func (_m *Accessory) Kind() string {
	ret := _m.Called()

	if len(ret) == 0 {
		panic("no return value specified for Kind")
	}

	var r0 string
	if rf, ok := ret.Get(0).(func() string); ok {
		r0 = rf()
	} else {
		r0 = ret.Get(0).(string)
	}

	return r0
}

// NewAccessory creates a new instance of Accessory. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewAccessory(t interface {
	mock.TestingT
	Cleanup(func())
}) *Accessory {
	mock := &Accessory{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}
