.progress-size-small {
    height: 0.5em !important;
}

.visibility-hidden {
    visibility: hidden;
}

.forgot-password-link {
    position: relative;
    line-height: 36px;
    font-size: 14px;
    float: right;
    top: -5px;
}

.tm-font {
    font-size: 14px !important;
    position: relative;
    top: -9px;
}

.bg{
    position: absolute;
    top: 60px; 
    right: 0;
    width: 100%; 
    height: 100%; 
    background-size: cover;
}

.login-wrapper {
    flex-wrap: wrap;

    .login {
        background:transparent;
    }
}

.reset-label {
    font-size:inherit;
    color:inherit;
    font-family: inherit;
}

.display-flex {
    display: flex;
    align-items: center;
    justify-content: space-between;
}

.pw-eye {
    margin-left: -20px;
    margin-right: 8px;
    cursor: pointer;
}

.pwd-input {
    padding-right: 26px;
}

.divider-container {
    display: flex;
    align-items: center;
    justify-content: space-between;
}

.divider {
    margin: 0;
    width: 8.5rem;
}
/* stylelint-disable no-descending-specificity */
.login {
    padding-bottom: 180px;
}

.back-icon {
    position: absolute;
    top: 1rem;
    display: flex;
    align-items: center;
    color: #007cbb;
    font-size: 16px;
    cursor: pointer;
}

.margin-left-3 {
    margin-left: 3px;
}

.title {
    position: absolute;
    top: 10rem
}

.login-group {
    width: 70%;
    position: absolute;
    top: 10rem
}

.more-info {
    text-align: right;
    padding-top: 2rem;
}

.oidc-login-button {
    text-transform: unset !important;
}
