// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import { NgModule } from '@angular/core';
import { SharedModule } from '../../../shared/shared.module';
import { ConfigurationComponent } from './config.component';
import { ConfigurationAuthComponent } from './auth/config-auth.component';
import { SystemSettingsComponent } from './system/system-settings.component';
import { RouterModule, Routes } from '@angular/router';
import { ConfigService } from './config.service';
import { SecurityComponent } from './security/security.component';

const routes: Routes = [
    {
        path: '',
        component: ConfigurationComponent,
        children: [
            {
                path: 'auth',
                component: ConfigurationAuthComponent,
            },
            {
                path: 'security',
                component: SecurityComponent,
            },
            {
                path: 'setting',
                component: SystemSettingsComponent,
            },
            {
                path: '',
                redirectTo: 'auth',
                pathMatch: 'full',
            },
        ],
    },
];
@NgModule({
    imports: [SharedModule, RouterModule.forChild(routes)],
    declarations: [
        ConfigurationComponent,
        ConfigurationAuthComponent,
        SystemSettingsComponent,
        SecurityComponent,
    ],
    providers: [ConfigService],
})
export class ConfigurationModule {}
