// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { DEFAULT_SUPPORTED_MIME_TYPES } from '../../../../../shared/units/utils';

@Injectable({
    providedIn: 'root',
})
export class AdditionsService {
    constructor(private http: HttpClient) {}

    getDetailByLink(
        link: string,
        shouldSetHeader: boolean,
        shouldReturnText: boolean
    ): Observable<any> {
        if (shouldReturnText) {
            return this.http.get(link, {
                observe: 'body',
                responseType: 'text',
            });
        }
        if (shouldSetHeader) {
            return this.http.get(link, {
                headers: {
                    'X-Accept-Vulnerabilities': DEFAULT_SUPPORTED_MIME_TYPES,
                },
            });
        }
        return this.http.get(link);
    }
}
