// Code generated by mockery v2.53.3. DO NOT EDIT.

package postprocessors

import (
	context "context"

	mock "github.com/stretchr/testify/mock"
)

// NativeScanReportConverter is an autogenerated mock type for the NativeScanReportConverter type
type NativeScanReportConverter struct {
	mock.Mock
}

// FromRelationalSchema provides a mock function with given fields: ctx, reportUUID, artifactDigest, reportSummary
func (_m *NativeScanReportConverter) FromRelationalSchema(ctx context.Context, reportUUID string, artifactDigest string, reportSummary string) (string, error) {
	ret := _m.Called(ctx, reportUUID, artifactDigest, reportSummary)

	if len(ret) == 0 {
		panic("no return value specified for FromRelationalSchema")
	}

	var r0 string
	var r1 error
	if rf, ok := ret.Get(0).(func(context.Context, string, string, string) (string, error)); ok {
		return rf(ctx, reportUUID, artifactDigest, reportSummary)
	}
	if rf, ok := ret.Get(0).(func(context.Context, string, string, string) string); ok {
		r0 = rf(ctx, reportUUID, artifactDigest, reportSummary)
	} else {
		r0 = ret.Get(0).(string)
	}

	if rf, ok := ret.Get(1).(func(context.Context, string, string, string) error); ok {
		r1 = rf(ctx, reportUUID, artifactDigest, reportSummary)
	} else {
		r1 = ret.Error(1)
	}

	return r0, r1
}

// ToRelationalSchema provides a mock function with given fields: ctx, reportUUID, registrationUUID, digest, reportData
func (_m *NativeScanReportConverter) ToRelationalSchema(ctx context.Context, reportUUID string, registrationUUID string, digest string, reportData string) (string, string, error) {
	ret := _m.Called(ctx, reportUUID, registrationUUID, digest, reportData)

	if len(ret) == 0 {
		panic("no return value specified for ToRelationalSchema")
	}

	var r0 string
	var r1 string
	var r2 error
	if rf, ok := ret.Get(0).(func(context.Context, string, string, string, string) (string, string, error)); ok {
		return rf(ctx, reportUUID, registrationUUID, digest, reportData)
	}
	if rf, ok := ret.Get(0).(func(context.Context, string, string, string, string) string); ok {
		r0 = rf(ctx, reportUUID, registrationUUID, digest, reportData)
	} else {
		r0 = ret.Get(0).(string)
	}

	if rf, ok := ret.Get(1).(func(context.Context, string, string, string, string) string); ok {
		r1 = rf(ctx, reportUUID, registrationUUID, digest, reportData)
	} else {
		r1 = ret.Get(1).(string)
	}

	if rf, ok := ret.Get(2).(func(context.Context, string, string, string, string) error); ok {
		r2 = rf(ctx, reportUUID, registrationUUID, digest, reportData)
	} else {
		r2 = ret.Error(2)
	}

	return r0, r1, r2
}

// NewNativeScanReportConverter creates a new instance of NativeScanReportConverter. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewNativeScanReportConverter(t interface {
	mock.TestingT
	Cleanup(func())
}) *NativeScanReportConverter {
	mock := &NativeScanReportConverter{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}
