.ml-5 {
  margin-left: 5px;
}

.dropdown-toggle {
  outline: none;
  padding-left: 0;
}

.backdrop-transparent {
  position: fixed;
  top: 0;
  bottom: 0;
  right: 0;
  left: 0;
  opacity: 0;
  z-index: 999;
}

.v-center {
  height: 48px;
  line-height: 48px;
}

.font-size-54 {
  font-size: 0.5417rem;
}

.flex-150 {
  flex: 0 0 150px;
}

.rule-name {
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
  font-size: 0.5417rem;
}

.color-97 {
  color: #979797;
}

.add-rule-btn {
  min-width: 120px;
  padding-left: 12px;
}
