.ml-20px {
    margin-left: 20px;
}

.action-bar {
    margin-top: 0;
    display: flex;
    align-items: center;
}

.datagrid-host {
    position: inherit;
}

:host::ng-deep.datagrid-spinner {
    top: 5rem !important;
    width: 62% !important;
    height: 32% !important;
}

:host::ng-deep.spinner {
    left: 14rem !important;
}
