.icon-tooltip {
  margin-top: 4px;
}

.padding-top-3 {
  padding-top: 3px;
}

.bottom-btn {
  text-align: right;
}

.width-238 {
  width: 238px;
}

.width-8rem {
  width: 8rem;
}

.clr-control-label {
  width: 10rem !important;
}
