// Variables for light theme should be defined here.
@import "../../node_modules/@clr/ui/clr-ui.min.css";

  
$mode-background-color: #fff;
$mode-background-color1: $mode-background-color;
$mode-background-color2: #fafafa;
$mode-background-color3: #eee;
$fill-color1: #000;
$mode-font-color1: #313131;
$mode-font-color3: $fill-color1;
$mode-font-color4: unset;
$mode-font-color5: #575757;
$mode-font-color2: #565656;
$mode-link-color1: blue;
$mode-link-color2: #0077b8;
$select-option-color: #1b2a32;
$select-back-color: $mode-background-color;
$label-form-color: $mode-background-color3;
$right-status-fill-color: #1d5100;
$light-color-green: $right-status-fill-color;
$color-ddd: #ddd;
$color-f2: #f2f2f2;
$color-657b83: #657b83;
$color-fdf6e3:  #fdf6e3;
$header-color: rgb(0 74 112);
$color-yaml:  #fdf6e3;
$filter-divider-bg-color: #ccc;
/* stylelint-disable */
$selectBox-option-hover-bg-color-start: #f5f5f5;
$selectBox-option-hover-bg-color-end: #e8e8e8;
$hbr-result-tip-histogram-inner-bg-color: #fff;
$harbor-icon-translate-x: 100%;
$harbor-icon-drop-shadow-x: -56px;
$command-input-color: none;
$command-input-bg-color: #ededed;
$label-hover-bg-color: #eee;
$nav-divider-bg-color: #000;
$label-color-input: #5d5d5d;
$central-block-loading-bg-color: rgb(255 255 255 / 50%);
$back-link-color: none;
$radio-button-checked: #0072a3;
$input-autofill-bg-color: #fafafa;
$input-autofill-color: #000;
$pull-command-icon-color: #007CBB;
$pull-command-icon-hover-color: #4aaed9;
$select-all-for-dropdown-color: #0072a3;
$normal-border-color: #6a7a81;
$text-color-job-service-dashboard: #0072a3;
$datagrid-numeric-filter-input-bg-color: unset;
$alert-info-bg-color: #107eb0;
@import "./common.scss";
