.content-wrapper {
    margin-top: 20px;

    .md-container {
        border: solid 1px #ddd;
    }

    .md-div {
        padding: 14px 14px 14px 0;

        .file {
            width: 100%;
            display: flex;
            align-items: center;
            justify-content: space-between;
        }
    }
}

.center {
    justify-content: center;
    align-items: center;
}
