.side-nav-override {
    box-shadow: 6px 0 0 0 #ccc;
}

.container-override {
    position: relative !important;
}

.content-container{
    position: relative;
}

.start-content-padding {
    padding: 0 !important;
    background-color: white;
    overflow-y: hidden !important;
}

.content-area-override {
    padding-left: 36px !important;
    padding-bottom: 36px !important;
    padding-right: 21px !important;
}

.nav-link-override {
    width: 126px !important;
}

clr-vertical-nav {
    ::ng-deep {
        .nav-content {
            display: flex;
            justify-content: space-between;
        }
    }
}

.font-size-13 {
    font-size: 13px;
}

.global-msg {
    position: absolute;
    top: 0;
    left: 2rem;
    right: 2rem;
    z-index: 600;
}
