$thirty-pixel: 26px;
$twenty-two-pixel: 18px;

.bar-wrapper {
    width: 144px;
    height: 12px;
}

.bar-state {
    text-align: center;

    .unknow-text {
        margin-left: -5px;
    }
}

.bar-state-chart {
    margin-top: 2px;

    .loop-height {
        height: 2px;
    }
}

.bar-state-error {
    position: relative;
    top: -4px;
}

.error-text {
    position: relative;
    top: 1px;
    margin-left: -5px;
    cursor: pointer;
}

.scanning-button {
    height: 24px;
    margin-top: 0;
    margin-bottom: 0;
    vertical-align: middle;
    top: -12px;
    position: relative;
}

.tip-wrapper {
    display: flex;
    align-items: center;
    color: #fff;
    text-align: center;
    font-size: 10px;
    height: 15px;
    line-height: 15px;
}

.bar-block-critical {
    background-color: #ff4d2e;
}

.bar-block-high {
    background-color: #ff8f3d;
}

.bar-block-medium {
    background-color: #ffce66;
}

.bar-block-low {
    background-color: #fff1ad;
}

.bar-block-none {
    background-color: #2ec0ff;
}

.bar-block-unknown {
    background-color: grey;
}

.bar-tooltip-font {
    font-size: 13px;
    color: #fff;
}

.bar-tooltip-font-title {
    font-weight: 600;
}

.bar-summary {
    margin-top: 12px;
    text-align: left;
}

.bar-scanning-time {
    margin-top: 12px;
}

.bar-summary-item {
    margin-top: 3px;
    margin-bottom: 3px;

    span {
        :nth-child(1) {
            width: $thirty-pixel;
            text-align: center;
            display: inline-block;
        }

        :nth-child(2) {
            width: 28px;
            display: inline-block;
        }
    }
}

.option-right {
    padding-right: 16px;
}

.refresh-btn {
    cursor: pointer;
    /* stylelint-disable */
    :hover {
        color: #007CBB;
    }
}

.tip-icon-medium {
    color: orange;
}


.tip-icon-low {
    color: #007CBB;
}

.font-color-green {
    color: green;
}

.bar-tooltip-font-larger {
    span {
        font-size: 16px;
        vertical-align: middle
    }
}

hr {
    border-bottom: 0;
    border-color: #aaa;
    margin: 6px -10px;
}

.font-weight-600 {
    font-weight: 600;
}

.result-row {
    position: relative;
}

.help-icon {
    margin-left: 3px;
}

.ml-3px {
    margin-left: 3px;
}

.w-360 {
    width: 360px !important;
}

.margin-left-5 {
    margin-left: 5px;
}

.margin-left-10 {
    margin-left: 10px;
}

.width-30 {
    width: $thirty-pixel;
}

.width-215 {
    width: 215px;
}

.width-150 {
    width: 150px;
}

.level-border>div{
    display: inline-flex;
    align-items: center;
    justify-items: center;
    border-radius: 50%;
    height: $thirty-pixel;
    width: $thirty-pixel;
    line-height: $thirty-pixel;
}

.inner {
    margin: auto;
    height: $twenty-two-pixel;
    width: $twenty-two-pixel;
    line-height: $twenty-two-pixel;
    font-size: 12px;
    font-weight: 600;
    border-radius: 50%;
    text-align: center;
}

.level-critical {
    background:#FF4D2E;
    color:#FF4D2E;
}

.level-high {
    background:#FF8F3D;
    color:#FF8F3D;
}

.level-medium {
    background-color: #FFCE66;
    color:#FFCE66;
}

.level-low {
    background: #FFF1AD;
    color:#FFF1AD;
}

.level-none {
    background-color: grey;
    color:grey;
}

.black-point-container {
    display: inline-block;
    position: relative;
    height: 6px;
}

.black-point {
    position: absolute;
    width: 4px;
    height: 4px;
    border-radius: 50%;
}

.font-weight-800 {
    font-weight: 800;
}

.color-green {
    color: green;
}

.tip-block {
    position: relative;
}

.font-size-14 {
    font-size: 14px;
}

.font-size-12 {
    font-size: 12px;
}

.margin-right-5 {
    margin-right: 5px;
}

.is-white {
    background-color: #fff;
}

.is-dark {
    background-color: #000;
}
