.flex {
  display: flex;
}

.label-critical {
  background:#ff4d2e;
  color:#000;
}

.label-danger {
  background:#ff8f3d!important;
  color:#000!important;
}

.label-medium {
  background-color: #ffce66;
  color:#000;
}

.label-low {
  background: #fff1ad;
  color:#000;
}

.label-none {
  background-color: #2ec0ff;
  color:#000;
}

.no-border {
  border: none;
}


.action-bar {
  display: flex;
  align-items: flex-end;
  justify-content: space-between;
}

.refresh-btn {
  margin-right: 2rem;
  cursor: pointer;

  &:hover {
    color: #007CBB;
  }
}

.repo-name {
  min-width: 10rem;
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}

.ellipsis {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}

.min-width {
  min-width: 9rem !important;
}

.label {
  min-width: 3rem;
}
