// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package volcenginecr

import "errors"

var (
	regionRegs = map[string]any{
		"(.*)-(cn-.*)": nil,
	}
	errListNamespaceResp    = errors.New("[VolcengineCR adapt] ListNamespaces resp nil")
	errListRepositoriesResp = errors.New("[VolcengineCR adapt] ListRepositories resp nil")
	errListTagsResp         = errors.New("[VolcengineCR adapt] ListTags resp nil")
	errPareseDigest         = errors.New("[VolcengineCR adapt]  fail to parse reference")
	errNilVolcCrClient      = errors.New("[volcengine-cr.createRepository] nil volcCr client")
)

const (
	MaxPageSize     int64 = 100
	concurrentLimit int   = 3
)
