// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Injectable } from '@angular/core';
import { SessionUser, SessionUserBackend } from '../entities/session-user';
import { clone } from '../units/utils';

@Injectable({
    providedIn: 'root',
})
export class SessionViewmodelFactory {
    constructor() {}
    // view model need
    getCurrentUser(currentUser: SessionUserBackend): SessionUser {
        return {
            user_id: currentUser.user_id,
            username: currentUser.username,
            email: currentUser.email,
            realname: currentUser.realname,
            role_name: currentUser.role_name,
            role_id: currentUser.role_id,
            comment: currentUser.comment,
            oidc_user_meta: currentUser.oidc_user_meta,
            has_admin_role:
                currentUser.admin_role_in_auth || currentUser.sysadmin_flag,
        };
    }
}
