
.mr-10 {
    margin-right:0.5rem;
}

.display-flex {
    display: flex;
    align-items:center;
}

.pos-inherit {
    position: inherit;
}

.clr-error {
    .clr-select-wrapper::after {
        right: 0.25rem !important;
    }
}

.input-width {
    width: 242px;
}

.mt-02 {
    margin-top: 0.2rem;
}

.width-164 {
    width: 164px;
}

.endpoint {
    display: inline-block;
    width: 72px;
}

.display-none {
    display: none
}

.space-between {
    display: flex;
    justify-content: space-between;
}

.go-link {
    line-height: 1rem;
    cursor: pointer;
}

.alert-label {
    color:red;
    font-size: 12px;
}

.width-182 {
    width: 182px;
}

.mt-05 {
    margin-top: 0.5rem;
}
