// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { HarborRouteReuseStrategy } from './harbor-route-reuse-strategy';
import { ActivatedRouteSnapshot } from '@angular/router';

describe('HarborRouteReuseStrategy', () => {
    let harborRouteReuseStrategy: HarborRouteReuseStrategy;
    beforeEach(() => {
        harborRouteReuseStrategy = new HarborRouteReuseStrategy();
    });
    it('should be created', () => {
        expect(harborRouteReuseStrategy).toBeTruthy();
    });
    it('shouldReuseRoute', () => {
        const future: ActivatedRouteSnapshot = new ActivatedRouteSnapshot();
        const curr: ActivatedRouteSnapshot = new ActivatedRouteSnapshot();
        expect(
            harborRouteReuseStrategy.shouldReuseRoute(future, curr)
        ).toBeTruthy();
    });
});
