// Code generated by mockery v2.53.3. DO NOT EDIT.

package v1

import (
	v1 "github.com/goharbor/harbor/src/pkg/scan/rest/v1"
	mock "github.com/stretchr/testify/mock"
)

// ClientPool is an autogenerated mock type for the ClientPool type
type ClientPool struct {
	mock.Mock
}

// Get provides a mock function with given fields: url, authType, accessCredential, skipCertVerify
func (_m *ClientPool) Get(url string, authType string, accessCredential string, skipCertVerify bool) (v1.Client, error) {
	ret := _m.Called(url, authType, accessCredential, skipCertVerify)

	if len(ret) == 0 {
		panic("no return value specified for Get")
	}

	var r0 v1.Client
	var r1 error
	if rf, ok := ret.Get(0).(func(string, string, string, bool) (v1.Client, error)); ok {
		return rf(url, authType, accessCredential, skipCertVerify)
	}
	if rf, ok := ret.Get(0).(func(string, string, string, bool) v1.Client); ok {
		r0 = rf(url, authType, accessCredential, skipCertVerify)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(v1.Client)
		}
	}

	if rf, ok := ret.Get(1).(func(string, string, string, bool) error); ok {
		r1 = rf(url, authType, accessCredential, skipCertVerify)
	} else {
		r1 = ret.Error(1)
	}

	return r0, r1
}

// NewClientPool creates a new instance of ClientPool. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewClientPool(t interface {
	mock.TestingT
	Cleanup(func())
}) *ClientPool {
	mock := &ClientPool{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}
