.form-group-label-override {
    font-size: 14px;
    font-weight: 400;
}
/* stylelint-disable */
.selectBox{
    position: absolute;
    width: 173px;
    height: auto;
    background-color: white;
    border: 1px solid rgb(0 0 0 / 15%);
    border-right-width: 2px;
    border-bottom-width: 2px;
    border-radius: 6px;
    box-shadow: 0 5px 10px rgb(0 0 0 / 20%);
    z-index: 100;
}

.selectBox ul li{
    list-style: none;
    padding: 3px 20px;
    cursor: pointer;
}

.selectBox ul li:hover{
    color: #262626;
    background-image: linear-gradient(180deg,#f5f5f5 0,#e8e8e8);
    background-repeat: repeat-x;
}

.modal-body{
    overflow: visible;
}


