.label-form {
    margin-bottom: -10px;
    padding-top: 0;
    margin-top: 20px;
    width: 100%;
    background-color: #eee;
    border: 1px solid #ccc;

    section {
      padding: .5rem 0;
    }
  
    section>label {
      margin-left: 20px;
    }
  /* stylelint-disable */
    .btnColor {
      margin: 0 !important;
      padding: 0 !important;
      width: 26px;
      height: 22px;
      min-width: 26px;
    }

    .colorPanel {
      span {
        margin: 5px 4px;
        width: 30px;
        height: 24px;
        text-align: center;
        line-height: 24px;
        font-size: 12px;
        border: 1px solid #A1A1A1;
      }
    }
  
    .dropdown-menu {
      display: inline-block;
      width: 166px;
      padding: 6px;

      .dropdown-item {
        display: inline-block;
        margin: 2px 4px;
        padding: 0;
        width: 30px;
        height: 24px;
        text-align: center;
        line-height: 24px;
        border: 0;
        color: white;
        font-size: 12px;
      }
    }
  
    .dropdown .dropdown-toggle.btn {
      padding: 0 !important;
    }

    .borderSty
    {
      border: 1px solid #A1A1A1 !important;
      line-height: 22px;
    }
}

::ng-deep {
  .clr-form-control {
    display: inherit;
  }
}

.position-ab {
  position: absolute;
}
