.option-left {
    padding-left: 16px;
    position: relative;
    top: 10px;
}

.option-right {
    padding-right: 16px;
}

.refresh-btn {
    cursor: pointer;
}

.refresh-btn:hover {
    color: #007CBB;
}

:host::ng-deep .btn-group-overflow .dropdown-toggle {
    line-height: 24px;
    height: 24px;
}
/* stylelint-disable */
.rightPos{
    position: absolute;
    z-index: 100;
    right: 35px;
    margin-top: 4px;
}

clr-datagrid {
    ::ng-deep clr-checkbox {
         position: inherit;
     }
 }

:host::ng-deep clr-checkbox.checkbox.datagrid-foot-select {
    position: inherit;
}

.content-space {
    margin-top: 28px;
    position: relative;
}

.datagrid-pt {
    padding-top: 0;
}

.dropdown-header {
    text-transform: none;
    font-size: 13px; 
}

