// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package api

import (
	"net/http"

	"github.com/goharbor/harbor/src/lib/log"
)

// Health ...
func Health(w http.ResponseWriter, _ *http.Request) {
	if err := WriteJSON(w, "healthy"); err != nil {
		log.Errorf("Failed to write response: %v", err)
		return
	}
}
