.list-rule {
    padding-bottom: 15px;
}

.text-alignment {
    vertical-align: text-bottom;
}

.min-width {
    min-width: 236px;
}

.col-width {
    min-width: 140px;
}

.status-width {
    min-width: 130px;
}

.icon-style {
    color: #C92100;
}

.margin-right-2px {
    margin-right: 2px;
}

.trigger {
    width: 180px;
}
